@@ -3,7 +3,7 @@
 #
 # ibus-table - The Tables engine for IBus
 #
-# Copyright (c) 2008-2008 Yu Yuwei <acevery@gmail.com>
+# Copyright (c) 2008-2009 Yu Yuwei <acevery@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,11 +27,11 @@ __all__ = (
 
 import os
 import ibus
-from ibus import Property
+#from ibus import Property
 from ibus import keysyms
 from ibus import modifier
 from ibus import ascii
-import tabsqlitedb
+#import tabsqlitedb
 import tabdict
 import re
 patt_edit = re.compile (r'(.*)###(.*)###(.*)')
@@ -102,7 +102,11 @@ class editor(object):
     def get_chinese_mode (self):
         '''Use LC_CTYPE in your box to determine the _chinese_mode'''
         try:
-            __lc = os.environ['LC_CTYPE'].split('.')[0].lower()
+            if os.environ.has_key('LC_CTYPE'):
+                __lc = os.environ['LC_CTYPE'].split('.')[0].lower()
+            else:
+                __lc = os.environ['LANG'].split('.')[0].lower()
+
             if __lc.find('zh_') == 0:
                 # this is a zh_XX
                 __place =__lc.split('_')[1]
@@ -453,7 +457,7 @@ class editor(object):
         else:
             # this is a system phrase haven't been used:
             attrs.append ( ibus.AttributeForeground (0x000000, 0, len(_phrase)) )
-        self._lookup_table.append_candidate ( _phrase + _tbks, attrs )
+        self._lookup_table.append_candidate ( ibus.Text(_phrase + _tbks, attrs) )
         self._lookup_table.show_cursor (False)
 
     def filter_candidates (self, candidates):
@@ -549,13 +553,13 @@ class editor(object):
                                 or only_one_last:
                             # because we use [!@#$%] to denote [12345]
                             # in py_mode, so we need to distinguish them
+                            ## old manner:
                             if self._py_mode:
                                 if self._chars[0][-1] in "!@#$%":
                                     self._chars[0].pop() 
                                     self._tabkey_list.pop()
                                     return True
 
-                            ## old manner:
                             if self._candidates[1]:
                                 self._candidates[0] = self._candidates[1]
                                 self._candidates[1] = []
@@ -780,12 +784,14 @@ class editor(object):
             return (False,u'')
         if self._t_chars :
             # user has input sth
+            istr = self.get_all_input_strings ()
             self.commit_to_preedit ()
             pstr = self.get_preedit_strings ()
+            #print "istr: ",istr
             self.clear()
-            return (True,pstr)
+            return (True,pstr,istr)
         else:
-            return (False,u'')
+            return (False,u'',u'')
     
     def one_candidate (self):
         '''Return true if there is only one candidate'''
@@ -863,7 +869,10 @@ class tabengine (ibus.EngineBase):
                 ]
         # some properties we will involved, Property is taken from scim.
         #self._setup_property = Property ("setup", _("Setup"))
-        self._direct_commit = False
+        try:
+            self._auto_commit = self.db.get_ime_property('auto_commit').lower() == u'true'
+        except:
+            self._auto_commit = False
         # the commit phrases length
         self._len_list = [0]
         # connect to SpeedMeter
@@ -876,6 +885,8 @@ class tabengine (ibus.EngineBase):
                     "org.ibus.table.SpeedMeter") 
         except:
             self._sm = None
+        self._sm_on = False
+        self._on = False
         self.reset ()
 
     def reset (self):
@@ -888,6 +899,9 @@ class tabengine (ibus.EngineBase):
         self._update_ui ()
     
     def do_destroy(self):
+        self.reset ()
+        self.focus_out ()
+        #self.db.sync_usrdb ()
         super(tabengine,self).do_destroy()
 
     def _init_properties (self):
@@ -899,13 +913,13 @@ class tabengine (ibus.EngineBase):
         self._punct_property = ibus.Property(u'punct')
         self._py_property = ibus.Property(u'py_mode')
         self._onechar_property = ibus.Property(u'onechar')
-        self._direct_commit_property = ibus.Property(u'dcommit')
+        self._auto_commit_property = ibus.Property(u'acommit')
         for prop in (self._status_property,
             self._letter_property,
             self._punct_property,
             self._py_property,
             self._onechar_property,
-            self._direct_commit_property
+            self._auto_commit_property
             #self._setup_property
             ):
             self.properties.append(prop)
@@ -958,12 +972,12 @@ class tabengine (ibus.EngineBase):
         else:
             self._onechar_property.set_icon ( u'%s%s' % (self._icon_dir, 'phrase.svg' ))
             self._onechar_property.set_tooltip ( _(u'Switch to single char mode') )
-        if self._direct_commit:
-            self._direct_commit_property.set_icon ( u'%s%s' % (self._icon_dir, 'dcommit.svg' ) ) 
-            self._direct_commit_property.set_tooltip ( _(u'Switch to normal commit mode, which use space to commit') ) 
+        if self._auto_commit:
+            self._auto_commit_property.set_icon ( u'%s%s' % (self._icon_dir, 'acommit.svg' ) ) 
+            self._auto_commit_property.set_tooltip ( _(u'Switch to normal commit mode, which use space to commit') ) 
         else:
-            self._direct_commit_property.set_icon ( u'%s%s' % (self._icon_dir, 'ncommit.svg' ) ) 
-            self._direct_commit_property.set_tooltip ( _(u'Switch to direct commit mode') ) 
+            self._auto_commit_property.set_icon ( u'%s%s' % (self._icon_dir, 'ncommit.svg' ) ) 
+            self._auto_commit_property.set_tooltip ( _(u'Switch to direct commit mode') ) 
         # the chinese_mode:
         if self.db._is_chinese:
             if self._editor._chinese_mode == 0:
@@ -1005,8 +1019,8 @@ class tabengine (ibus.EngineBase):
             self._editor.r_shift ()
         elif property == u'onechar':
             self._editor._onechar = not self._editor._onechar
-        elif property == u'dcommit':
-            self._direct_commit = not self._direct_commit
+        elif property == u'acommit':
+            self._auto_commit = not self._auto_commit
         elif property == u'letter':
             self._full_width_letter [self._mode] = not self._full_width_letter [self._mode]
         elif property == u'punct':
@@ -1024,7 +1038,7 @@ class tabengine (ibus.EngineBase):
         '''Update Preedit String in UI'''
         _str = self._editor.get_preedit_strings ()
         if _str == u'':
-            self.update_preedit (u'', None, 0, False)
+            super(tabengine, self).update_preedit_text(ibus.Text(u'',None), 0, False)
         else:
             attrs = ibus.AttrList()
             res = patt_edit.match (_str)
@@ -1055,7 +1069,7 @@ class tabengine (ibus.EngineBase):
             attrs.append(ibus.AttributeUnderline(ibus.ATTR_UNDERLINE_SINGLE, 0, len(_str)))
 
 
-            self.update_preedit (_str, attrs, self._editor.get_caret(), True)
+            super(tabengine, self).update_preedit_text(ibus.Text(_str, attrs), self._editor.get_caret(), True)
     
     def _update_aux (self):
         '''Update Aux String in UI'''
@@ -1064,9 +1078,9 @@ class tabengine (ibus.EngineBase):
             attrs = ibus.AttrList([ ibus.AttributeForeground(0x9515b5,0, len(_ic)) ])
             #attrs = [ scim.Attribute(0,len(_ic),scim.ATTR_FOREGROUND,0x5540c1)]
 
-            self.update_aux_string (_ic, attrs, True)
+            super(tabengine, self).update_auxiliary_text(ibus.Text(_ic, attrs), True)
         else:
-            self.hide_aux_string ()
+            self.hide_auxiliary_text()
             #self.update_aux_string (u'', None, False)
 
     def _update_lookup_table (self):
@@ -1083,15 +1097,16 @@ class tabengine (ibus.EngineBase):
         self._update_aux ()
 
     def add_string_len(self, astring):
-        try:
-            self._sm.Accumulate(len(astring))
-        except:
-            pass
+        if self._sm_on:
+            try:
+                self._sm.Accumulate(len(astring))
+            except:
+                pass
     
     def commit_string (self,string):
         self._editor.clear ()
         self._update_ui ()
-        super(tabengine,self).commit_string ( string )
+        super(tabengine,self).commit_text ( ibus.Text(string) )
         self._prev_char = string[-1]
 
     def _convert_to_full_width (self, c):
@@ -1142,12 +1157,12 @@ class tabengine (ibus.EngineBase):
 
         return False
     
-    def process_key_event(self, keyval, is_press, state):
+    def process_key_event(self, keyval, keycode, state):
         '''Process Key Events
         Key Events include Key Press and Key Release,
         modifier means Key Pressed
         '''
-        key = KeyEvent (keyval, is_press, state)
+        key = KeyEvent(keyval, state & modifier.RELEASE_MASK == 0, state)
         # ignore NumLock mask
         key.mask &= ~modifier.MOD2_MASK
 
@@ -1242,7 +1257,21 @@ class tabengine (ibus.EngineBase):
             return True
         # Match direct commit mode switch hotkey
         if self._match_hotkey (key, keysyms.slash, modifier.CONTROL_MASK):
-            self.property_activate ( u"dcommit" )
+            self.property_activate ( u"acommit" )
+            return True
+        
+        # Match Chinese mode shift
+        if self._match_hotkey (key, keysyms.semicolon, modifier.CONTROL_MASK):
+            self.property_activate ( u"cmode" )
+            return True
+        
+        # Match speedmeter shift
+        if self._match_hotkey (key, keysyms.apostrophe, modifier.CONTROL_MASK):
+            self._sm_on = not self._sm_on
+            if self._sm_on:
+                self._sm.Show ()
+            else:
+                self._sm.Hide ()
             return True
         
         # Ignore key release event now :)
@@ -1251,11 +1280,12 @@ class tabengine (ibus.EngineBase):
 
         if self._editor.is_empty ():
             # we have not input anything
-            if key.code <= 127 and (not key.mask & modifier.ALT_MASK + modifier.CONTROL_MASK):
+            if key.code <= 127 and ( unichr(key.code) not in self._valid_input_chars ) \
+                    and (not key.mask & modifier.ALT_MASK + modifier.CONTROL_MASK):
                 if key.code == keysyms.space:
                     self.commit_string (cond_letter_translate (unichr (key.code)))
                     return True
-                if ascii.ispunct (key.code) and ( unichr(key.code) not in self._valid_input_chars ) :
+                if ascii.ispunct (key.code):
                     self.commit_string (cond_punct_translate (unichr (key.code)))
                     return True
                 if ascii.isdigit (key.code):
@@ -1274,15 +1304,15 @@ class tabengine (ibus.EngineBase):
             self.commit_string (commit_string)
             return True
         
-        #elif key.code in (keysyms.Down, keysyms.KP_Down) :
-        #    res = self._editor.arrow_down ()
-        #    self._update_ui ()
-        #    return res
+        elif key.code in (keysyms.Down, keysyms.KP_Down) :
+            res = self._editor.arrow_down ()
+            self._update_ui ()
+            return res
         
-        #elif key.code in (keysyms.Up, keysyms.KP_Up):
-        #    res = self._editor.arrow_up ()
-        #    self._update_ui ()
-        #    return res
+        elif key.code in (keysyms.Up, keysyms.KP_Up):
+            res = self._editor.arrow_up ()
+            self._update_ui ()
+            return res
         
         elif key.code in (keysyms.Left, keysyms.KP_Left) and key.mask & modifier.CONTROL_MASK:
             res = self._editor.control_arrow_left ()
@@ -1341,7 +1371,7 @@ class tabengine (ibus.EngineBase):
             if sp_res[0]:
                 self.commit_string (sp_res[1])
                 self.add_string_len(sp_res[1])
-                self.db.check_phrase (sp_res[1])
+                self.db.check_phrase (sp_res[1], sp_res[2])
             else:
                 if sp_res[1] == u' ':
                     self.commit_string (cond_letter_translate (u" "))
@@ -1359,7 +1389,7 @@ class tabengine (ibus.EngineBase):
         elif unichr(key.code) in self._valid_input_chars or \
                 ( self._editor._py_mode and \
                     unichr(key.code) in u'abcdefghijklmnopqrstuvwxyz!@#$%' ):
-            if self._direct_commit and ( len(self._editor._chars[0]) == self._ml \
+            if self._auto_commit and ( len(self._editor._chars[0]) == self._ml \
                     or len (self._editor._chars[0]) in self.db.pkeylens ):
                 # it is time to direct commit
                 sp_res = self._editor.space ()
@@ -1367,7 +1397,7 @@ class tabengine (ibus.EngineBase):
                 if sp_res[0]:
                     self.commit_string (sp_res[1])
                     self.add_string_len(sp_res[1])
-                    self.db.check_phrase (sp_res[1])
+                    self.db.check_phrase (sp_res[1],sp_res[2])
             
             res = self._editor.add_input ( unichr(key.code) )
             if not res:
@@ -1380,21 +1410,22 @@ class tabengine (ibus.EngineBase):
                 if sp_res[0]:
                     self.commit_string (sp_res[1] + key_char)
                     self.add_string_len(sp_res[1])
-                    self.db.check_phrase (sp_res[1])
+                    self.db.check_phrase (sp_res[1],sp_res[2])
                     return True
                 else:
                     self.commit_string ( key_char )
                     return True
             else:
-                if self._direct_commit and self._editor.one_candidate () and \
-                        len(self._editor._chars[0]) == self._ml:
+                if self._auto_commit and self._editor.one_candidate () and \
+                        (len(self._editor._chars[0]) == self._ml \
+                            or not self.db._is_chinese):
                     # it is time to direct commit
                     sp_res = self._editor.space ()
                     #return (whethercommit,commitstring)
                     if sp_res[0]:
                         self.commit_string (sp_res[1])
                         self.add_string_len(sp_res[1])
-                        self.db.check_phrase (sp_res[1])
+                        self.db.check_phrase (sp_res[1], sp_res[2])
                         return True
 
             self._update_ui ()
@@ -1411,6 +1442,7 @@ class tabengine (ibus.EngineBase):
             return res
         
         elif key.code >= keysyms._1 and key.code <= keysyms._9 and self._editor._candidates[0]:
+            input_keys = self._editor.get_all_input_strings ()
             res = self._editor.number (key.code - keysyms._1)
             if res:
                 o_py = self._editor._py_mode
@@ -1421,7 +1453,7 @@ class tabengine (ibus.EngineBase):
                     self._refresh_properties ()
                     self._update_ui ()
                 # modify freq info
-                self.db.check_phrase (commit_string)
+                self.db.check_phrase (commit_string, input_keys)
             return True
         
         elif key.code <= 127:
@@ -1440,13 +1472,17 @@ class tabengine (ibus.EngineBase):
     
     # below for initial test
     def focus_in (self):
-        self.register_properties (self.properties)
-        self._refresh_properties ()
-        self._update_ui ()
-        try:
-            self._sm.Show()
-        except:
-            pass
+        if self._on:
+            self.register_properties (self.properties)
+            self._refresh_properties ()
+            self._update_ui ()
+            try:
+                if self._sm_on:
+                    self._sm.Show ()
+                else:
+                    self._sm.Hide ()
+            except:
+                pass
     
     def focus_out (self):
         try:
@@ -1459,6 +1495,8 @@ class tabengine (ibus.EngineBase):
             self._sm.Reset()
         except:
             pass
+        self._on = True
+        self.focus_in()
 
     def disable (self):
         self.reset()
@@ -1466,6 +1504,8 @@ class tabengine (ibus.EngineBase):
             self._sm.Hide()
         except:
             pass
+        self._on = False
+
 
     def lookup_table_page_up (self):
         if self._editor.page_up ():
