@@ -48,6 +48,7 @@ class tabsqlitedb:
         self.deparse = tabdict.deparse
         self._add_phrase_sqlstr = ''
         self.old_phrases=[]
+        self.ime_property_cache = {}
         
         if filename:
             # now we are creating db
@@ -69,10 +70,10 @@ class tabsqlitedb:
             print 'encountering error when init db'
             pass
         # create IME property table
-        sqlstr = 'CREATE TABLE IF NOT EXISTS main.ime (attr TEXT, val TEXT);' 
-        self.db.executescript( sqlstr )
+        self.db.executescript('CREATE TABLE IF NOT EXISTS main.ime (attr TEXT, val TEXT);')
         # make sure we have values in ime table.
-        if not self.db.execute('SELECT * FROM main.ime;').fetchall():
+        if not self.db.execute('SELECT val FROM main.ime \
+            WHERE attr="name";').fetchall():
             ime_keys={'name':'',
                       'name.zh_cn':'',
                       'name.zh_hk':'',
@@ -81,7 +82,7 @@ class tabsqlitedb:
                       'uuid':'%s' % uuid.uuid4(),
                       'serial_number':'%s' % time.strftime('%Y%m%d'),
                       'icon':'ibus-table.svg',
-                      'credit':'GPL',
+                      'license':'LGPL',
                       'languages':'',
                       'valid_input_chars':'abcdefghijklmnopqrstuvwxyz',
                       'max_key_length':'4',
@@ -96,7 +97,10 @@ class tabsqlitedb:
                       'user_can_define_phrase':'FALSE',
                       'pinyin_mode':'FALSE',
                       'dynamic_adjust':'FALSE',
+                      'auto_commit':'false',
                       #'no_check_chars':u'',
+                      'description':'A IME under IBus Table',
+                      'layout':'us',
                       'rules':''}
                       #'rules':'ce2:p11+p12+p21+p22;ce3:p11+p21+p22+p31;ca4:p11+p21+p31+p41'}
             # inital the attribute in ime table, which should be updated from mabiao
@@ -109,8 +113,8 @@ class tabsqlitedb:
         self._is_chinese = self.is_chinese()
         # for fast add word
         self._set_add_phrase_sqlstr()
-        #(MLEN, CLEN, M0, M1, M2, M3, M4, PHRASE, FREQ, USER_FREQ) = range (0,10)
-        self._pt_index = ['mlen','clen']
+        #(ID, MLEN, CLEN, M0, M1, M2, M3, M4, CATEGORY, PHRASE, FREQ, USER_FREQ) = range (0,12)
+        self._pt_index = ['id', 'mlen', 'clen']
         for i in range(self._mlen):
             self._pt_index.append ('m%d' %i)
         if self._is_chinese:
@@ -159,7 +163,7 @@ class tabsqlitedb:
                 desc = self.get_database_desc (user_db)
                 if desc == None :
                     self.init_user_db (user_db)
-                elif desc["version"] != "0.2":
+                elif desc["version"] != "0.4":
                     new_name = "%s.%d" %(user_db, os.getpid())
                     print >> stderr, "Can not support the user db. We will rename it to %s" % new_name
                     self.old_phrases = self.extra_user_phrases( user_db )
@@ -188,13 +192,21 @@ class tabsqlitedb:
             self.db.execute ('ATTACH DATABASE "%s" AS user_db;' % user_db)
         self.create_tables ("user_db")
         if self.old_phrases:
-            self.old_phrases =\
-                    map(lambda x: [self.parse_phrase_to_tabkeys(x[0])]\
-                    + list(x) , self.old_phrases)
-            self.add_phrases (self.old_phrases, 'user_db')
+            # (mlen, phrase, freq, user_freq)
+            # the phrases will be deparse again, and then be added     
+            # the characters will be discard :(
+            #chars = filter (lambda x: x[0] == 1, self.old_phrases)
+            # print chars
+            phrases = filter (lambda x: x[0] > 1, self.old_phrases)
+            phrases = map(lambda x: [self.parse_phrase_to_tabkeys(x[1])]\
+                    + list(x[1:]) , phrases)
+ 
+            map (self.u_add_phrase,phrases)
+            self.db.commit ()
+
 
         # try create all tables in user database
-        self.create_indexes ("user_db")
+        self.create_indexes ("user_db",commit=False)
         self.generate_userdb_desc ()
         
         # attach mudb for working process
@@ -205,9 +217,9 @@ class tabsqlitedb:
     def update_phrase (self, entry, database='user_db'):
         '''update phrase freqs'''
         #print entry
-        _con = [ entry[-1] ] + list(entry[0:2+entry[0]]) + [entry[-3]]
+        _con = [ entry[-1] ] + list(entry[1:3+entry[1]]) + [entry[-3]]
         #print _con
-        _condition = u''.join( map(lambda x: 'AND m%d = ? ' % x, range(entry[0]) )    )
+        _condition = u''.join( map(lambda x: 'AND m%d = ? ' % x, range(entry[1]) )    )
         #print _condition
         sqlstr = 'UPDATE %s.phrases SET user_freq = ? WHERE mlen = ? AND clen = ? %s AND phrase = ?;' % (database, _condition)
         #print sqlstr
@@ -219,17 +231,19 @@ class tabsqlitedb:
         # we need to update the user_db
         #print 'sync userdb'
         mudata = self.db.execute ('SELECT * FROM mudb.phrases;').fetchall()
+        #print mudata
         data_u = filter ( lambda x: x[-2] in [1,-3], mudata)
         data_a = filter ( lambda x: x[-2]==2, mudata)
         data_n = filter ( lambda x: x[-2]==-2, mudata)
         #print data_a
-        data_a = map (lambda x: (u''.join ( map(self.deparse, x[2:2+x[0]])),x[-3],0,x[-1] ), data_a)
-        data_n = map (lambda x: (u''.join ( map(self.deparse, x[2:2+x[0]])),x[-3],-1,x[-1] ), data_n)
+        data_a = map (lambda x: (u''.join ( map(self.deparse, x[3:3+x[1]])),x[-3],0,x[-1] ), data_a)
+        data_n = map (lambda x: (u''.join ( map(self.deparse, x[3:3+x[1]])),x[-3],-1,x[-1] ), data_n)
         #print data_u
         map (self.update_phrase, data_u)
         #print self.db.execute('select * from user_db.phrases;').fetchall()
         map (self.u_add_phrase,data_a)
         map (self.u_add_phrase,data_n)
+        self.db.commit ()
     
     def is_chinese (self):
         __lang = self.get_ime_property ('languages')
@@ -271,7 +285,8 @@ class tabsqlitedb:
             self.db.execute ( sqlstr )
 
         # create phrase table (mabiao)
-        sqlstr = 'CREATE TABLE IF NOT EXISTS %s.phrases ( mlen INTEGER, clen INTEGER, ' % database
+        sqlstr = 'CREATE TABLE IF NOT EXISTS %s.phrases (id INTEGER PRIMARY KEY AUTOINCREMENT,\
+                mlen INTEGER, clen INTEGER, ' % database
         #for i in range(self._mlen):
         #    sqlstr += 'm%d INTEGER, ' % i 
         sqlstr += ''.join ( map (lambda x: 'm%d INTEGER, ' % x, range(self._mlen)) )
@@ -294,11 +309,13 @@ class tabsqlitedb:
             else:
                 #print '"',attr,'"'," didn't in ime property now!"
                 pass
+        # then flush previous cache
+        self.ime_property_cache = {}
         # we need to update some self variables now.
         self._mlen = int (self.get_ime_property ('max_key_length' ))
         self._is_chinese = self.is_chinese()
         self._set_add_phrase_sqlstr()
-        self._pt_index = ['mlen','clen']
+        self._pt_index = ['id', 'mlen', 'clen']
         for i in range(self._mlen):
             self._pt_index.append ('m%d' %i)
         if self._is_chinese:
@@ -389,7 +406,7 @@ class tabsqlitedb:
     
     def u_add_phrase (self,nphrase):
         '''Add a phrase to userdb'''
-        self.add_phrase (nphrase,database='user_db')
+        self.add_phrase (nphrase,database='user_db',commit=False)
 
     def _set_add_phrase_sqlstr(self):
         '''Create the sqlstr for add phrase according to self._mlen.'''
@@ -544,7 +561,8 @@ class tabsqlitedb:
         sqlstr = '''
             CREATE TABLE tmp AS SELECT * FROM %(database)s.phrases;
             DELETE FROM %(database)s.phrases;
-            INSERT INTO %(database)s.phrases SELECT * FROM tmp ORDER BY %(tabkeystr)s mlen ASC, freq DESC;
+            INSERT INTO %(database)s.phrases SELECT * FROM tmp ORDER BY
+            %(tabkeystr)s mlen ASC, user_freq DESC, freq DESC, id ASC;
             DROP TABLE tmp;
             CREATE TABLE tmp AS SELECT * FROM %(database)s.goucima;
             DELETE FROM %(database)s.goucima;
@@ -575,7 +593,7 @@ class tabsqlitedb:
         self.db.executescript (sqlstr)
         self.db.commit()
     
-    def create_indexes(self, database):
+    def create_indexes(self, database, commit=True):
         sqlstr = '''
             DROP INDEX IF EXISTS %(database)s.goucima_index_z;
             CREATE INDEX IF NOT EXISTS %(database)s.goucima_index_z ON goucima (zi);
@@ -586,7 +604,8 @@ class tabsqlitedb:
 
         sqlstr_t = '''
             DROP INDEX IF EXISTS %(database)s.phrases_index_p;
-            CREATE INDEX IF NOT EXISTS %(database)s.phrases_index_p ON phrases (%(tabkeystr)s mlen ASC, freq DESC);
+            CREATE INDEX IF NOT EXISTS %(database)s.phrases_index_p ON phrases
+            (%(tabkeystr)s mlen ASC, freq DESC, id ASC);
             DROP INDEX IF EXISTS %(database)s.phrases_index_i;
             CREATE INDEX IF NOT EXISTS %(database)s.phrases_index_i ON phrases (phrase, mlen ASC);
             ''' 
@@ -598,10 +617,12 @@ class tabsqlitedb:
         else:
             sqlstr = sqlstr_t % {'database':database,'tabkeystr':tabkeystr }
         self.db.executescript (sqlstr)
-        self.db.commit()
+        if commit:
+            self.db.commit()
     
     def compare (self,x,y):
-        return cmp (x[0],y[0]) or -(cmp (x[-1],y[-1])) or -(cmp (x[-2],y[-2]))
+        return cmp (x[1], y[1]) or -(cmp (x[-1], y[-1])) \
+                or -(cmp (x[-2], y[-2])) or (cmp (x[0], y[0]))
 
     def select_words( self, tabkeys, onechar=False, bitmask=0 ):
         '''
@@ -637,7 +658,7 @@ class tabsqlitedb:
             SELECT * FROM user_db.phrases WHERE mlen < %(mk)d %(condition)s 
             UNION ALL
             SELECT * FROM mudb.phrases WHERE mlen < %(mk)d %(condition)s )
-            ORDER BY mlen ASC, user_freq DESC, freq DESC;''' % { 'mk':_len+x_len, 'condition':_condition}
+            ORDER BY mlen ASC, user_freq DESC, freq DESC, id ASC;''' % { 'mk':_len+x_len, 'condition':_condition}
             # we have redefine the __int__(self) in class tabdict.tab_key to return the key id, so we can use map to got key id :)
             _tabkeys = map(int,tabkeys[:_len])
             _tabkeys += _tabkeys + _tabkeys
@@ -656,7 +677,8 @@ class tabsqlitedb:
         #searchres = map ( lambda res: res[-2] and [ True, [(res[:-2],[res[:-1],res[-1:]])] ]\
         #        or [ False, [(res[:-2] , [res[:-1],res[-1:]])] ] \
         #        , result )
-        searchres = map ( lambda res: [ int(res[-2]), int(res[-1]), [(res[:-2],[res[:-1],res[-1:]])] ], result)
+        searchres = map ( lambda res: [ int(res[-2]), int(res[-1]),
+            [(res[1:-2],[res[:-1],res[-1:]])] ], result)
         # for sysdb
         reslist=filter( lambda x: not x[1], searchres )
         map (lambda x: sysdb.update(x[2]), reslist)
@@ -708,11 +730,27 @@ class tabsqlitedb:
         #    _condition += 'AND p%d = ? ' % i
         _condition += ''.join ( map (lambda x: 'AND p%d = ? ' %x, range(_len)) )
         # you can increase the x in _len + x to include more result, but in the most case, we only need one more key result, so we don't need the extra overhead :)
-        sqlstr = '''SELECT * FROM main.pinyin WHERE plen < %(mk)d  %(condition)s 
-        ORDER BY plen ASC, freq DESC;''' % { 'mk':_len+3, 'condition':_condition}
-        # we have redefine the __int__(self) in class tabdict.tab_key to return the key id, so we can use map to got key id :)
-        _tabkeys = map(int,tabkeys[:_len])
-        result = self.db.execute(sqlstr, _tabkeys).fetchall()
+        # here we need make sure that the 3 <= plen <=7,
+        # so , if if _len < 3, than we start from 3; 
+        #   if _len >= 3, we start from _len + 1;
+        #   if _len = 7, we start from _len
+        if _len < 7:
+            if _len < 3:
+                x_len = 3
+            else:
+                x_len = _len + 1
+        else:
+            x_len = _len
+        
+        while x_len < 8:
+            sqlstr = '''SELECT * FROM main.pinyin WHERE plen < %(mk)d  %(condition)s 
+                ORDER BY plen ASC, freq DESC;''' % { 'mk':x_len, 'condition':_condition}
+            # we have redefine the __int__(self) in class tabdict.tab_key to return the key id, so we can use map to got key id :)
+            _tabkeys = map(int,tabkeys[:_len])
+            result = self.db.execute(sqlstr, _tabkeys).fetchall()
+            if len(result) > 0:
+                break
+            x_len += 1
         #self.db.commit()
         return result[:]
 
@@ -720,13 +758,14 @@ class tabsqlitedb:
         '''get IME property from database, attr is the string of property,
         which should be str.lower() :)
         '''
-        sqlstr = 'SELECT val FROM main.ime WHERE attr = ?' 
-        _result = self.db.execute( sqlstr, (attr,)).fetchall()
-        #self.db.commit()
-        if _result:
-            return _result[0][0]
-        else:
-            return None
+        if not attr in self.ime_property_cache:
+            sqlstr = 'SELECT val FROM main.ime WHERE attr = ?' 
+            _result = self.db.execute( sqlstr, (attr,)).fetchall()
+            if _result:
+                self.ime_property_cache[attr] = _result[0][0]
+            else:
+                self.ime_property_cache[attr] = None
+        return self.ime_property_cache[attr]
 
     def get_phrase_table_index (self):
         '''get a list of phrase table columns name'''
@@ -737,8 +776,7 @@ class tabsqlitedb:
             sqlstring = 'CREATE TABLE IF NOT EXISTS user_db.desc (name PRIMARY KEY, value);'
             self.db.executescript (sqlstring)
             sqlstring = 'INSERT OR IGNORE INTO user_db.desc  VALUES (?, ?);'
-            self.db.execute (sqlstring, ('version', '0.2'))
-            self.db.execute (sqlstring, ('id', str(uuid.uuid4 ())))
+            self.db.execute (sqlstring, ('version', '0.4'))
             sqlstring = 'INSERT OR IGNORE INTO user_db.desc  VALUES (?, DATETIME("now", "localtime"));'
             self.db.execute (sqlstring, ("create-time", ))
             self.db.commit ()
@@ -849,7 +887,10 @@ class tabsqlitedb:
             tabres = self.parse_phrase (phrase) [2:-1]
         except:
             tabres = None
-        tabkeys= u''.join ( map(self.deparse, tabres) )
+        if tabres:
+            tabkeys= u''.join ( map(self.deparse, tabres) )
+        else:
+            tabkeys= u''
         return tabkeys
 
     def check_phrase (self,phrase,tabkey=None,database='main'):
@@ -878,35 +919,56 @@ class tabsqlitedb:
             except:
                 # if we don't have goucima:
                 return
-        if tabkey == None:
+        if (not tabkey) or len(tabkey) > self._mlen :
             sqlstr = '''SELECT * FROM (SELECT * FROM main.phrases WHERE phrase = ?
             UNION ALL SELECT * FROM user_db.phrases WHERE phrase = ?
             UNION ALL SELECT * FROM mudb.phrases WHERE phrase = ?)
-            ORDER BY user_freq DESC, freq DESC
+            ORDER BY user_freq DESC, freq DESC, id ASC;
             ''' 
             result = self.db.execute(sqlstr, (phrase,phrase,phrase)).fetchall()
         else:
             # we are using this to check whether the tab-key and phrase is in db
+            #print "tabkey: ", tabkey
             tabks = self.parse (tabkey)
+            #print "tabks: ", tabks
             tabkids = tuple( map(int,tabks) )
             condition = ' and '.join( map(lambda x: 'm%d = ?' % x, range( len(tabks) )) )
-            sqlstr = '''SELECT * FROM %(database)s.phrases WHERE phrase = ? and %(cond)s;''' % {'database':database, 'cond':condition}
-            result = self.db.execute(sqlstr, (phrase,)+tabkids ).fetchall()
-            return bool(result)
+            sqlstr = '''SELECT * FROM 
+            (
+                SELECT * FROM main.phrases WHERE phrase = ? and %(cond)s
+                UNION ALL SELECT * FROM user_db.phrases WHERE phrase = ? and %(cond)s
+                UNION ALL SELECT * FROM mudb.phrases WHERE phrase = ? and %(cond)s
+            )
+            ORDER BY user_freq DESC, freq DESC, id ASC;
+            ''' % {'cond':condition}
+            #print sqlstr
+            result = self.db.execute(sqlstr, ((phrase,)+tabkids)*3 ).fetchall()
+            if not bool(result):
+                sqlstr = '''SELECT * FROM (SELECT * FROM main.phrases WHERE phrase = ?
+                UNION ALL SELECT * FROM user_db.phrases WHERE phrase = ?
+                UNION ALL SELECT * FROM mudb.phrases WHERE phrase = ?)
+                ORDER BY user_freq DESC, freq DESC, id ASC;
+                ''' 
+                result = self.db.execute(sqlstr, (phrase,phrase,phrase)).fetchall()
 
         sysdb = {}
         usrdb = {}
         mudb = {}
-        searchres = map ( lambda res: [ int(res[-2]), int(res[-1]), [(res[:-2],[res[:-1],res[-1]])] ], result)
+        #print "result is: ", result 
+        searchres = map ( lambda res: [ int(res[-2]), int(res[-1]),
+            [(res[1:-2],[res[:-1],res[-1]])] ], result)
         # for sysdb
         reslist=filter( lambda x: not x[1], searchres )
         map (lambda x: sysdb.update(x[2]), reslist)
+        #print "sysdb is ", sysdb
         # for usrdb
         reslist=filter( lambda x: ( x[0] in [0,-1] ) and x[1], searchres )
         map (lambda x: usrdb.update(x[2]), reslist)
+        #print "usrdb is ", usrdb 
         # for mudb
         reslist=filter( lambda x: (x[0] not in [0,-1])  and x[1], searchres )
         map (lambda x: mudb.update(x[2]), reslist)
+        #print "mudb is ", mudb
         
         tabkey = ''
         if len(phrase) >=2:
@@ -1004,8 +1066,8 @@ class tabsqlitedb:
             if result:
                 for _res in result:
                     tabkey = u''
-                    for i in range ( int ( _res[0] ) ):
-                        tabkey += self.deparse ( _res[2+i] )
+                    for i in range ( int ( _res[1] ) ):
+                        tabkey += self.deparse ( _res[3+i] )
                     codes.append(tabkey)
         except:
             import traceback
@@ -1015,11 +1077,11 @@ class tabsqlitedb:
     def remove_phrase (self,phrase,database='user_db'):
         '''Remove phrase from database, default is from user_db
         phrase should be the a row of select * result from database
-        Like (mlen,clen,m0,m1,m2,m3,phrase,freq,user_freq)
+        Like (id, mlen,clen,m0,m1,m2,m3,phrase,freq,user_freq)
         '''
         _ph = list(phrase[:-2])
         _condition = ''    
-        for i in range(_ph[0]):
+        for i in range(_ph[1]):
             _condition += 'AND m%d = ? ' % i
         nn =_ph.count(None)
         if nn:
@@ -1029,24 +1091,24 @@ class tabsqlitedb:
             msqlstr= 'SELECT * FROM %(database)s.phrases WHERE mlen = ? and clen = ? %(condition)s AND category = ? AND phrase = ? ;' % { 'database':database, 'condition':_condition }
         else:
             msqlstr= 'SELECT * FROM %(database)s.phrases WHERE mlen = ? and clen = ? %(condition)s AND phrase = ? ;' % { 'database':database, 'condition':_condition }
-        if self.db.execute(msqlstr, _ph).fetchall():
+        if self.db.execute(msqlstr, _ph[1:]).fetchall():
             if self._is_chinese:
                 sqlstr = 'DELETE FROM %(database)s.phrases WHERE mlen = ? AND clen =? %(condition)s AND category = ? AND phrase = ?  ;' % { 'database':database, 'condition':_condition }
             else:
                 sqlstr = 'DELETE FROM %(database)s.phrases WHERE mlen = ? AND clen =? %(condition)s AND phrase = ?  ;' % { 'database':database, 'condition':_condition }
-            self.db.execute(sqlstr,_ph)
+            self.db.execute(sqlstr,_ph[1:])
             self.db.commit()
 
         if self._is_chinese:
             msqlstr= 'SELECT * FROM mudb.phrases WHERE mlen = ? and clen = ? %(condition)s AND category = ? AND phrase = ? ;' % { 'condition':_condition }
         else:
             msqlstr= 'SELECT * FROM mudb.phrases WHERE mlen = ? and clen = ? %(condition)s AND phrase = ? ;' % { 'condition':_condition }
-        if self.db.execute(msqlstr, _ph).fetchall():
+        if self.db.execute(msqlstr, _ph[1:]).fetchall():
             if self._is_chinese:
                 sqlstr = 'DELETE FROM mudb.phrases WHERE mlen = ? AND clen =? %(condition)s AND category = ? AND phrase = ?  ;' % {  'condition':_condition }
             else:
                 sqlstr = 'DELETE FROM mudb.phrases WHERE mlen = ? AND clen =? %(condition)s AND phrase = ?  ;' % {  'condition':_condition }
-            self.db.execute(sqlstr,_ph)
+            self.db.execute(sqlstr,_ph[1:])
             self.db.commit()
 
     def extra_user_phrases(self, udb, only_defined=False):
@@ -1056,10 +1118,16 @@ class tabsqlitedb:
         except:
             return None
         if only_defined:
-            _phrases = db.execute("select phrase, freq, user_freq\
-                    from phrases where freq=-1;").fetchall()
+            _phrases = db.execute(\
+                    "SELECT clen, phrase, freq, sum(user_freq)\
+                    FROM phrases \
+                    WHERE freq=-1 AND mlen != 0 \
+                    GROUP BY clen,phrase;").fetchall()
         else:
-            _phrases = db.execute("select phrase, freq, user_freq\
-                    from phrases;").fetchall()
+            _phrases = db.execute(\
+                    "SELECT clen, phrase, freq, sum(user_freq)\
+                    FROM phrases\
+                    WHERE mlen !=0 \
+                    GROUP BY clen,phrase;").fetchall()
         db.commit()
         return _phrases[:]
